from letschatty.models.chat.chat import Chat
from letschatty.models.company.assets.ai_agents.chatty_ai_agent import ChattyAIAgent
from letschatty.models.company.empresa import EmpresaModel
from letschatty.models.chat.flow_link_state import SmartFollowUpState
from letschatty.models.company.assets.ai_agents.follow_up_strategy import FollowUpStrategy
from letschatty.services.ai_agents import cot_follow_up_examples
from letschatty.services.ai_agents.relevant_context_picker import RelevantContextPicker
from letschatty.services.ai_agents.context_builder import ContextBuilder
from letschatty.models.utils.custom_exceptions.custom_exceptions import HumanInterventionRequired, PostponeFollowUp
from letschatty.models.company.assets.ai_agents.chatty_ai_mode import ChattyAIMode
from datetime import datetime, timedelta
from zoneinfo import ZoneInfo
import json
import logging

logger = logging.getLogger("SmartFollowUpContextBuilder")


class SmartFollowUpContextBuilder(ContextBuilder):

    @staticmethod
    def build_prompt(chat_smart_follow_up_assigned: SmartFollowUpState, follow_up_strategy: FollowUpStrategy, company_info: EmpresaModel, chatty_ai_agent: ChattyAIAgent, chat: Chat, mode_in_chat: ChattyAIMode) -> str:

        """
        Builds a comprehensive prompt for the AI agent to make intelligent follow-up decisions.

        Args:
            chat_smart_follow_up_assigned: Current follow-up state for the chat
            follow_up_strategy: Strategy configuration to use

        Returns:
            Complete prompt string for the AI agent
        """
        if not chat.is_free_conversation_active and not follow_up_strategy.templates_allowed:
            mode_in_chat = ChattyAIMode.SUGGESTIONS
        context = ContextBuilder.common_prompt(agent=chatty_ai_agent, mode_in_chat=mode_in_chat, company_info=company_info)
        context += f"In this particular case, you've been tasked with deciding whether and how to follow up with customers who haven't responded. Your goal is to be helpful, contextually aware, and respectful while driving business objectives."
        context += ContextBuilder.relevant_contexts(chatty_ai_agent.contexts, chat)
        context += ContextBuilder.unbreakable_rules(agent=chatty_ai_agent)
        context += ContextBuilder.control_triggers(agent=chatty_ai_agent)
        context += SmartFollowUpContextBuilder.follow_up_strategy_context(chat_smart_follow_up_assigned, follow_up_strategy)
        context += f"And here are some real business applications of the follow up strategy: {ContextBuilder.relevant_examples(follow_up_strategy.examples, chat)}"
        return context


    @staticmethod
    def follow_up_strategy_context(chat_smart_follow_up_assigned: SmartFollowUpState, follow_up_strategy: FollowUpStrategy) -> str:
        if follow_up_strategy.maximum_consecutive_follow_ups == 0:
            raise HumanInterventionRequired("There's a 0 limit for the maximum consecutive follow ups, human intervention is required")
        if follow_up_strategy.maximum_follow_ups_to_be_executed == 0:
            raise HumanInterventionRequired("There's a 0 limit for the total follow ups to be executed, human intervention is required")
        if follow_up_strategy.maximum_consecutive_follow_ups <= chat_smart_follow_up_assigned.consecutive_count:
            raise HumanInterventionRequired(f"Se alcanzó el máximo de seguimientos consecutivos: {follow_up_strategy.maximum_consecutive_follow_ups}.")
        if follow_up_strategy.maximum_follow_ups_to_be_executed <= chat_smart_follow_up_assigned.total_followups_sent:
            raise HumanInterventionRequired(f"We've reached the maximum total follow ups to be executed limit of {follow_up_strategy.maximum_follow_ups_to_be_executed}, human intervention is required")

        if datetime.now(ZoneInfo('UTC')).weekday() >= 5:
            raise PostponeFollowUp("The follow up is postponed because it's a weekend")

        next_followup_number = chat_smart_follow_up_assigned.consecutive_count + 1
        next_interval_hours = follow_up_strategy.get_interval_for_followup(next_followup_number)

        context = f"The follow up strategy is: {follow_up_strategy.name}. Your specific instructions and goals are: {follow_up_strategy.instructions_and_goals}"
        context += f"The consecutive follow ups sent are: {chat_smart_follow_up_assigned.consecutive_count} / {follow_up_strategy.maximum_consecutive_follow_ups}"
        context += f"The total follow ups sent are: {chat_smart_follow_up_assigned.total_followups_sent} / {follow_up_strategy.maximum_follow_ups_to_be_executed}"
        context += SmartFollowUpContextBuilder.next_follow_up(chat_smart_follow_up_assigned, follow_up_strategy)
        context += SmartFollowUpContextBuilder.decision_actions_available()
        context += SmartFollowUpContextBuilder.load_training_examples_from_json()
        return context

    @staticmethod
    def next_follow_up(chat_smart_follow_up_assigned: SmartFollowUpState, follow_up_strategy: FollowUpStrategy) -> str:
        next_call = datetime.now(ZoneInfo('UTC')) + timedelta(hours=follow_up_strategy.get_interval_for_followup(chat_smart_follow_up_assigned.consecutive_count + 1))
        next_interval_hours = follow_up_strategy.get_interval_for_followup(chat_smart_follow_up_assigned.consecutive_count + 1)
        context = f"As far as the next call, if you decide to send / suggest messages, the standard interval would be: {next_interval_hours} hour, so the next call would be: {next_call.isoformat()}. Unless you determine that the situation requires a different interval, in which case you should set the next call time accordingly. If you decide to skip, the next call time up to you based on the context and reason for skipping."
        if follow_up_strategy.only_on_weekdays and next_call.weekday() >= 5:
            context += f"Keep in mind that you are {'supposed' if follow_up_strategy.only_on_weekdays else 'not supposed'} to send messages on weekends."
        return context

    @staticmethod
    def decision_actions_available() -> str:
        decisions = """
        ## DECISION ACTIONS AVAILABLE
        ### SEND
        - Use when: Customer needs gentle nudging, asked question without response, sent proposal/quote
        - **Requirements**: Message must be helpful, contextual, and advance the conversation. Always focus on adding value to the conversation and the client, not just asking for a response. Try to get to know the client better, double click on their interests and needs regarding the product or service asked. If you can, continue the conversation."

        ### SKIP
        - Skip this follow-up cycle but keep the sequence active
        - Use when: Customer indicated they need time ("La semana que viene te confirmo", "Lo veo más tarde cuadno salgo de trabajar")", inappropriate timing.
        - **Set custom next_call time** based on context (e.g., Monday if they "la semana que viene", or a few hours if they "lo veo más tarde")

        ### SUGGEST
        - Recommend follow-up to human agent instead of sending automatically
        - Use when: Either the ai mode is SUGGESTIONS, or if AI mode is COPILOT and you determine that the situation requires human touch, complex negotiation, customer seems frustrated, high-value deal needing personal attention
        - Human will decide whether to send your suggested message or craft their own

        ### REMOVE
        - Completely stop all automatic follow-ups for this chat
        - Use when: Customer explicitly said "don't contact me", conversation naturally concluded (deal closed/lost) and no further follow ups are needed or you've already sent the maximum number of follow ups (total or consecutive)

        """
        return decisions


    @staticmethod
    def load_training_examples_from_json() -> str:
        """
        Loads follow-up training examples from JSON file and formats them into a prompt section.

        Args:
            json_file_path: Path to the JSON file containing training examples

        Returns:
            Formatted string with examples and instructions to focus on reasoning patterns

        Raises:
            FileNotFoundError: If JSON file doesn't exist
            json.JSONDecodeError: If JSON is malformed
        """

        try:
            from .cot_follow_up_examples import follow_up_training_examples
            # Load JSON file
            if not follow_up_training_examples:
                return ""

            # Group examples by action type
            examples_by_action = {
                "send": [],
                "skip": [],
                "suggest": [],
                "remove": []
            }

            for example in follow_up_training_examples:
                title_lower = example.get('title', '').lower()
                if "send" in title_lower:
                    action_type = "send"
                elif "skip" in title_lower:
                    action_type = "skip"
                elif "suggest" in title_lower:
                    action_type = "suggest"
                elif "remove" in title_lower:
                    action_type = "remove"
                else:
                    continue  # Skip examples that don't match action types

                examples_by_action[action_type].append(example)

            # Build training section
            training_section = """
    ## TRAINING EXAMPLES FOR DECISION-MAKING

    **CRITICAL INSTRUCTION:** These examples use GENERIC business scenarios.

    **✅ What to learn:**
    - The REASONING PATTERNS in chain_of_thought sections
    - TIMING considerations and context analysis
    - WHEN to use each action based on situation
    - HOW conversation context affects decisions

    **❌ What NOT to copy:**
    - Specific message wording or content
    - Business details or industry context
    - Exact phrasing (create your own appropriate for THIS business)

    **Your task:** Apply the same decision logic but craft messages using your actual strategy instructions and business context.

    ---

    """

            # Format examples by action type
            for action, action_examples in examples_by_action.items():
                if not action_examples:
                    continue

                training_section += f"### {action.upper()} ACTION EXAMPLES\n\n"

                for example in action_examples:
                    training_section += f"**{example.get('title', 'Untitled Example')}**\n\n"

                    content = example.get('content', [])
                    for element in content:
                        element_type = element.get('type', '')
                        element_content = element.get('content', '')

                        if element_type == "user":
                            training_section += f"👤 Customer: {element_content}\n\n"
                        elif element_type == "ai":
                            training_section += f"🤖 Agent: {element_content}\n\n"
                        elif element_type == "chain_of_thought":
                            training_section += f"🧠 **REASONING:** {element_content}\n\n"

                    training_section += "---\n\n"

            return training_section

        except FileNotFoundError as e:
            # Log the error but don't break the prompt building
            logger.warning(f"Training examples file not found: {e}")
            return ""
        except json.JSONDecodeError as e:
            # Log the error but don't break the prompt building
            logger.warning(f"Invalid JSON in training examples file: {e}")
            return ""
        except Exception as e:
            # Log unexpected errors but don't break the prompt building
            logger.warning(f"Error loading training examples: {e}")
            return ""
